package io.tswf.transmutations.common.reader

import groovy.transform.CompileStatic
import io.tswf.transmutations.common.exception.InvalidClassReferenceAnnotationException
import org.objectweb.asm.Type
import org.objectweb.asm.tree.AnnotationNode

@CompileStatic
class ClassReferenceReader {

    public static final String CLASS_DESCRIPTOR_PARAMETER_NAME = "descriptor"
    public static final String CLASS_PARAMETER_NAME = "value"

    static Type getTypeFromClassRefAnnotation(AnnotationNode annotationNode) {
        findTypeFromClassRefAnnotation(annotationNode)
                ?.ensureNotNull { new InvalidClassReferenceAnnotationException("There is no class reference parameters on ${annotationNode.desc}!") }
    }

    static Type findTypeFromClassRefAnnotation(AnnotationNode annotationNode) {
        def descriptorValue = annotationNode.findValueByName(CLASS_DESCRIPTOR_PARAMETER_NAME, String)
        def classValue = annotationNode.findValueByName(CLASS_PARAMETER_NAME, Type)

        if (descriptorValue != null && classValue != null) {
            throw new InvalidClassReferenceAnnotationException("The class reference annotation '${annotationNode.desc}' must contain only one from these parameters: ['${CLASS_DESCRIPTOR_PARAMETER_NAME}', '${CLASS_PARAMETER_NAME}']")
        }

        if (descriptorValue != null) {
            return Type.getType(descriptorValue)
        }

        if (classValue != null) {
            return classValue
        }

        return null
    }
}
